#include <iostream>
#include <cstdlib>
#include <cstring>
#include <string>
#include <stdexcept>
#include <fstream>
#include <getopt.h>
#include <libscstring.hpp>
#include "version.hpp"

static void print_help() {
    std::cout << "usage: valid-utf8 [-h]\n";
    std::cout << "  -h, --help    show this help text and exit\n";
    std::cout << "  --version     show version number and exit\n";
}

static int check_file(std::istream& file) {
    std::string line;
    while (std::getline(file, line)) {
        if (!sc::is_valid_utf8(line)) return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}

int main(int argc, char * argv[]) {
    try {
        int opt_char, opt_val;
        struct option long_options[] = {
            {"help", no_argument, nullptr, 'h'},
            {"version", no_argument, &opt_val, 1},
            {nullptr, 0, nullptr, 0}
        };
        while ((opt_char = getopt_long(argc, argv, "h", long_options, nullptr)) != -1) {
            switch (opt_char) {
                case 0: {
                    // handle long-only options here
                    switch (opt_val) {
                        case 1:
                            std::cout << valid_utf8_version() << std::endl;
                            return EXIT_SUCCESS;
                    }
                    break;
                }
                case 'h':
                    print_help();
                    return EXIT_SUCCESS;
                case '?':
                    throw std::runtime_error("unrecognized option");
            }
        }
        if (optind == argc) {
            return check_file(std::cin);
        } else if (optind == argc - 1) {
            std::ifstream file {argv[optind]};
            if (!file) throw std::runtime_error {"could not open file"};
            return check_file(file);
        }
        for (int i = optind; i < argc; ++i) {
            try {
                // process file argv[i]
                std::ifstream file {argv[i]};
                if (!file) throw std::runtime_error {std::string {"could not open file: "} + argv[i]};
                int ret = check_file(file);
                if (ret == EXIT_FAILURE) throw std::runtime_error {std::string {"not valid utf8: "} + argv[i]};
            } catch (const std::runtime_error& ex) {
                std::cerr << "valid-utf8: " << ex.what() << '\n';
            }
        }
    } catch (const std::exception& ex) {
        std::cerr << "valid-utf8: " << ex.what() << '\n';
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}
